import datetime
from typing import List

class basic_session(object): #Session object
    """A basic session object class."""
    def __init__(self, ENDPOINT:str, game:dict, session_id:int, metadata:str, total_places:int, timestamp:str):
        self.ENDPOINT = ENDPOINT
        
        self.game_dict_ = game
        self.session_id_ = session_id
        self.metadata_ = metadata
        self.total_places_ = total_places
        self.timestamp_ = timestamp

    from . import _game_

    @property
    def game(self) -> _game_.game: #Returns game object.
        """Returns game object countaining info about the game."""
        return self._game_.game(self.game_dict_)

    @property
    def id(self) -> str:
        """Returns id of session."""
        return self.session_id_

    @property
    def metadata(self) -> str:
        """The metadata of the session's game."""
        return self.metadata_

    @property
    def total_places(self) -> int:
        """The total amount of placement slots in the session's game."""
        return self.total_places_

    @property
    def datetime(self) -> datetime.datetime: #Returns date of sesstion in python datetime object.
        """Returns python datetime object of the date and time the session's game was created."""
        return datetime.datetime.strptime(self.timestamp_, '%Y-%m-%d %H:%M:%S.%f')

    # Aliases
    timestamp = datetime

class session(basic_session): #Class for full sessions data.
    """A full session object class. This includes a list of the players in the session."""
    def __init__(self, ENDPOINT:str, game_dict: dict, session_id: int, metadata: str, total_places: int, timestamp: dict, players:list):
        super().__init__(ENDPOINT, game_dict, session_id, metadata, total_places, timestamp)

        self.players_list = players

    from . import _player_

    @property
    def players(self) -> List[_player_.basic_player]: #Returns list of players as player objects.
        """Returns list of players who were in the session's game as basic player objects."""
        players_list = []
        for player_ in self.players_list:
            players_list.append(self._player_.basic_player(player_["player"]["player_id"], player_["player"]["uuid"], player_["player"]["username"]))

        return players_list




from __future__ import annotations
import datetime
from typing import List

class MCFPlayer(object):
    def __init__(self, data:dict):
        self.uid_:int = data["uid"]
        self.has_nova_account_:bool = data["has_nova_account"]
        self.nova_account_name_:str|None = data["nova_account_name"]
        self.username_:str = data["username"]
        self.uuid_:str = data["uuid"]
        self.team_number_:int = data["team_number"]
        self.score_:int = data["score"]
        self.kills_:int = data["kills"]

    @property
    def id(self):
        """Returns the MCF id of the player."""
        return self.uid_

    @property
    def has_nova_account(self):
        """Returns True/False if the player is linked to a novauniverse account."""
        return self.has_nova_account_

    @property
    def nova_account_name(self):
        """Returns the name of the nova universe account this player is linked to."""
        return self.nova_account_name_
    
    @property
    def username(self):
        """Returns the player's mojang username."""
        return self.username_

    @property
    def uuid(self):
        """Returns the player's mojang uuid."""
        return self.uuid_

    @property
    def team_number(self):
        """Returns the number of the team the player is in."""
        return self.team_number_

    @property
    def score(self):
        """Returns the total score this player achived."""
        return self.score_

    @property
    def kills(self):
        """Returns the amount of kills this player got."""
        return self.kills_

    uid = id
    """Returns the MCF id of the player."""

    name = username
    """Returns the player's mojang username."""

class BasicMCFTeam(object):
    def __init__(self, team_number:int, team_score:int):
        self.team_number_ = team_number
        self.team_score_ = team_score

    @property
    def team_number(self):
        """Returns the number of this team."""
        return self.team_number_

    @property
    def team_score(self):
        """Returns the total score of this team."""
        return self.team_score_

class ExtendedMCFTeam(BasicMCFTeam):
    def __init__(self, team_players:list, team_number:int, team_score:int):
        self.players_:list = team_players
        super().__init__(team_number, team_score)

    @property
    def players(self) -> List[MCFPlayer]:
        """Returns list of player objects of the players in that team."""
        player_list = []
        for player in self.players_:
            player_list.append(MCFPlayer(player))

        return player_list

class MCF(object):
    """This class represents a MCF tournament session/week."""

    def __init__(self, data:dict):
        self.data = data

        self.id_:int = data["id"]
        self.date_:str = data["date"]
        self.display_name_:str = data["display_name"]
        self.winner_team_id_:int = data["winner_team_id"]
        self.teams_:list = data["teams"]
        self.players_:list = data["players"]

    @property
    def id(self):
        return self.id_

    @property
    def date(self) -> datetime.datetime:
        return datetime.datetime.strptime(self.date_, '%Y-%m-%d')

    @property
    def display_name(self):
        """The display name of the MCF session."""
        return self.display_name_

    @property
    def winner_team_id(self):
        """The team number of the winning team."""
        return self.winner_team_id_

    @property
    def winner_team(self) -> ExtendedMCFTeam:
        """Returns team object of the winning team."""
        for team in self.teams:
            if team.team_number == self.winner_team_number:
                return team

    @property
    def teams(self) -> List[ExtendedMCFTeam]:
        """Returns list of all teams that played in this MCF."""
        teams_list = []
        for team in self.teams_:

            # Find the players of this team.
            team_players_list = []
            for player in self.players_:
                if player["team_number"] == team["team_number"]:
                    team_players_list.append(player)

            teams_list.append(ExtendedMCFTeam(team_players_list, team["team_number"], team["team_score"]))

        return teams_list

    winner_team_number = winner_team_id


from __future__ import annotations
from . import endpoints

import requests
import json

from .. import info

api_name = "NOVA API"
headers = {'User-Agent': (str(info.PACKAGE_NAME))[0:-2]}
error_format = f"\u001b[31m{api_name}" + ": [{}] {}\u001b[0m"

class NovaAPIError(Exception):
    pass

def request(url:str):
    """Makes request to api then returns data in json."""
    #Make request.
    response = requests.get(url, headers=headers)
    data:dict = response.json()
    
    try: # Nova Universe API error catcher.
        if data["success"] == False:
            error = data["error"]
            message = data["message"]

            raise NovaAPIError(error_format.format(error, message))
    except KeyError:
        pass

    try:
        return data["data"]
    except KeyError:
        return data

def request_list(url:str):
    """Makes request to api then returns data in json."""
    #Make request.
    response = requests.get(url, headers=headers)
    data:list = response.json()

    return data

def update(object, data_name):
    """Requests for specific pieces of data from the API. (Used for LIVE object attributes.)"""
    if not object.ENDPOINT == None:
        data = request(object.ENDPOINT)
        return data[f"{data_name}"]
    else:
        return None


class URLs:
    def __init__(self):
        self.base_url = "https://novauniverse.net/api"

        # Player
        self.player_stats_url = "/stats/player/{}"
        
        # Sessions
        self.session_stats_url = "/stats/session/by_id/{}"

        # Server Stats
        self.players_online_url = "/players/online/"
        self.basic_server_stats_url = "/stats/basic"
        self.extended_server_stats_url = "/stats/extended"

        # Tournament System validate license key.
        self.license_tournament_system_url = "/license/tournament_system/{}"

        # MCF Tournament
        self.mcf_result_url = "/mcf/result"

        # Internal Mojang API
        self.internal_mojang_api_url = "/private/mojang"
        

    def player_stats(self, player_uuid):
        return self.base_url + self.player_stats_url.format(player_uuid)

    def session_stats(self, session_id):
        return self.base_url + self.session_stats_url.format(session_id)

    def players_online(self):
        return self.base_url + self.players_online_url

    def basic_server_stats(self):
        return self.base_url + self.basic_server_stats_url

    def extended_server_stats(self):
        return self.base_url + self.extended_server_stats_url

    def mcf_result(self):
        return self.base_url + self.mcf_result_url

    def license_tournament_system(self, key:str):
        return self.base_url + self.license_tournament_system_url.format(key)

    def name_to_uuid(self, ign:str):
        return self.base_url + self.internal_mojang_api_url + f"/name_to_uuid/{ign}"


from . import API, objects

def license(key:str):
    """Returns information about a license key."""

    url = API.endpoints.URLs().license_tournament_system(key)
    plain_data = API.request(url)

    return objects._license_.License(plain_data["owner"], plain_data["expires_at"], plain_data["is_demo"], plain_data["is_active"])


from . import API, objects

def server():
    """Returns stats about the server. (E.g. player count, server count...)"""

    url = API.endpoints.URLs().basic_server_stats()
    url_2 = API.endpoints.URLs().players_online()
    plain_data = API.request(url)
    players_list = API.request(url_2)["players"]

    return objects._server_.BasicServer(url, url_2, plain_data["cached"], players_list)



from typing import List
from . import API, objects

def mcf() -> List[objects._mcf_.MCF]:
    """Returns list of MCF tournament results."""

    url = API.endpoints.URLs().mcf_result()
    plain_data = API.request_list(url)
    
    mcf_session_list = []
    for mcf_session in plain_data:
        mcf_session_list.append(objects._mcf_.MCF(mcf_session))

    return mcf_session_list



"""
Nova Universe (Python API Wrapper)

Copyright (c) 2022-present (Dev Goldy)
"""

from . import _find_, info, _server_, _keys_, _license_key_validation_, _mcf_

# Methods
Player = _find_.player
Session = _find_.session
Server = _server_.server
License = _license_key_validation_.license
Mcf = _mcf_.mcf

KEYS = _keys_


VER = 1.2
STAGE = ("alpha", 1)

VERSION = f"v{VER}-{STAGE[0]}-{STAGE[1]}" #Full version string.

PACKAGE_NAME = f"NovaUniverse.py ({VERSION}) (Made By Dev Goldy) \n"

FOOTER = "\nCopyright (c) 2022-present (Dev Goldy)"


import novauniverse as nova

"""
player = nova.Player("THEGOLDENPRO")

print(player.is_online)
"""

"""
license = nova.License(key=nova.KEYS.TOURNAMENT_DEMO_KEY)

print(license.owner)
"""

mcf_results = nova.Mcf()
for mcf in mcf_results:
    wining_players = mcf.winner_team.players
    print(f"[MCF: {mcf.display_name}] Winner: {wining_players[0].name} and {wining_players[1].name}")


from datetime import datetime

class License(object):
    def __init__(self, owner:str, expires_at:str, is_demo:str, is_active:str):
        self._owner = owner
        self._expires_at = expires_at
        self._is_demo = is_demo
        self._is_active = is_active

    @property
    def owner(self) -> str:
        """Returns the owner of the license key."""
        return self._owner

    @property
    def expires_at(self) -> datetime:
        """Returns the date the key expires as python datetime object."""
        return datetime.strptime(self._expires_at, '%Y-%m-%d %H:%M:%S')

    @property
    def is_demo(self) -> bool:
        if self._is_demo == "true": return True
        else: return False

    @property
    def is_active(self) -> bool:
        if self._is_active == "true": return True
        else: return False


import datetime
from typing import List

from .. import API

class basic_player(object):
    def __init__(self, id:int, uuid:str, username:str):
        self.id_ = id
        self.uuid_ = uuid
        self.username_ = username

    @property
    def id(self) -> str: #Returns Nova Universe id of player.
        """The Nova Universe id of the player."""
        return self.id_

    @property
    def uuid(self) -> str: #Returns Mojang uuid of player.
        """The Mojang uuid of the player."""
        return self.uuid_

    @property
    def username(self) -> str:
        """The “in game name”(ign) of the player."""
        return self.username_

    # Aliases
    name = username

class player(basic_player): #Player Object
    def __init__(self, ENDPOINT:str, id:int, uuid:str, username:str, first_join:dict, last_join:dict, is_online:int, sessions:list):
        super().__init__(id, uuid, username)
        self.ENDPOINT = ENDPOINT

        self.first_join_ = first_join
        self.last_join_ = last_join
        self.is_online_ = is_online
        self.sessions_ = sessions

    from . import _session_

    class player_session(_session_.basic_session): #Class for player's game sessions.
        def __init__(self, ENDPOINT:str, game: dict, session_id: int, metadata: str, total_places: int, player_placement:int, timestamp: dict):
            super().__init__(ENDPOINT, game, session_id, metadata, total_places, timestamp)

            self.player_placement_ = player_placement

        @property
        def player_placement(self) -> int:
            return self.player_placement_

    @property
    def first_join(self) -> datetime.datetime:
        """Returns datetime object of date and time the player first joined the Nova Universe network."""
        return datetime.datetime.strptime(self.first_join_, '%Y-%m-%d %H:%M:%S.%f')

    @property
    def last_join(self) -> datetime.datetime:
        """Returns datetime object of date and time the player last joined the Nova Universe network."""
        return datetime.datetime.strptime(self.last_join_, '%Y-%m-%d %H:%M:%S.%f')

    @property
    def is_online(self) -> bool: #Returns True/False if the player is currently present on the network.
        """Returns True/False if the player is currently present on the network."""
        if int(API.update(self, "is_online")) == 1: return True
        else: return False

    @property
    def sessions(self) -> List[player_session]: #Returns list of game sessions as session objects.
        """Returns list of sessions the player was in as player session objects"""
        sessions_list = []
        for session_ in self.sessions_:
            sessions_list.append(self.player_session(None, session_["game"], session_["session_id"], session_["metadata"], session_["total_places"], 
            session_["player_placement"], session_["timestamp"]))

        return sessions_list


from . import API
from . import objects

def player(player_name:str=None, player_uuid:str=None) -> objects._player_.player:
    """Finds the player and returns player object."""

    if player_uuid == None:
        if not player_name == None:
            player_uuid = player_name_to_uuid(player_name)
        else:
            raise Exception("Neither player name or player uuid was passed through the 'player object'. Please pass one of them in.")

    url = API.endpoints.URLs().player_stats(player_uuid)
    plain_data = API.request(url)

    return objects._player_.player(url, plain_data["id"], plain_data["uuid"], plain_data["username"], plain_data["first_join_timestamp"]["date"], plain_data["last_join_timestamp"]["date"], plain_data["is_online"], plain_data["sessions"])

def session(session_id:str) -> objects._session_.session:
    """Finds the game session and returns full session object."""

    url = API.endpoints.URLs().session_stats(session_id)
    plain_data = API.request(url)

    return objects._session_.session(url, plain_data["game"], plain_data["session_id"], plain_data["metadata"], plain_data["total_places"], plain_data["timestamp"]["date"], plain_data["players"])


def player_name_to_uuid(player_name:str):
    return (API.request(API.endpoints.URLs().name_to_uuid(player_name)))["full_uuid"]

"""
def short_to_full_uuid(short_uuid:str): #Converts short uuid to full uuid.
    return short_uuid[:8] + "-" + short_uuid[8:12] +  "-" + short_uuid[12:16] +  "-" + short_uuid[16:20] +  "-" + short_uuid[20:]
"""


from datetime import datetime
from typing import List
from .. import API

class BasicServer(object):
    """Class for basic server stats."""
    def __init__(self, ENDPOINT:str, ENDPOINT_2:str, cached:bool, players_online:dict):
        self.ENDPOINT = ENDPOINT
        self.ENDPOINT_2 = ENDPOINT_2

        self._cached = cached

        self.players_online_ = players_online

    from . import _player_

    class online_player(_player_.basic_player):
        def __init__(self, ENDPOINT: str, uuid: str, username: str, server_name:str, server_type_id:str, server_type_name:str, server_type_display_name:str):
            id = None # I don't have an easy and quick way to grab the player's id, due to it not being in this section of the api.
            super().__init__(id, uuid, username)

            self.server_name_ = server_name
            self.server_type_id_ = server_type_id
            self.server_type_name_ = server_type_name
            self.server_type_display_name_ = server_type_display_name

    @property
    def player_count(self) -> int:
        """The amount of players on the network right now. (LIVE)"""
        return int(API.update(self, "player_count"))

    @property
    def server_count(self) -> int:
        """The amount of servers online on the network right now. (LIVE)"""
        return int(API.update(self, "server_count"))

    @property
    def cached(self) -> bool:
        """Has this data been cached by the API or not."""
        if self._cached == "true": return True
        else: return False

    @property
    def localtime(self) -> datetime:
        """The local time at Zeeraa's house... uMm, returns as python datetime object. (LIVE)"""
        return datetime.strptime(API.update(self, "localtime")["date"], '%Y-%m-%d %H:%M:%S.%f')

    @property
    def timezone(self) -> str:
        """The name of the timezone the API server is in."""
        return str(API.update(self, "localtime")["timezone"])

    @property
    def players(self) -> List[online_player]:
        """Returns list of players online as player object."""
        players_list = []
        for player_ in self.players_online_:
            players_list.append(self.online_player(self.ENDPOINT_2, player_["uuid"], player_["username"], player_["server_name"], 
            player_["server_type_id"], player_["server_type_name"], player_["server_type_display_name"]))

        return players_list

    players_online = players
    online_players = players
    
    datetime = localtime
    time = localtime

class Server(object):
    """Class for extended server stats."""
    def __init__(self, ENDPOINT:str, cached:bool):
        super().__init__(ENDPOINT, cached)